@matheusbsbs / matheus-bagliano / commits / 70a9fef240
Fork 0

grep: move driver pre-load out of critical section

In builtin/grep.c:add_work() we pre-load the userdiff drivers before adding the grep_source in the todo list. This operation is currently being performed after acquiring the grep_mutex, but as it's already thread-safe, we don't need to protect it here. So let's move it out of the critical section which should avoid thread contention and improve performance. Running[1] `git grep --threads=8 abcd[02] HEAD` on chromium's repository[2], I got the following mean times for 30 executions after 2 warmups: Original | 6.2886s -------------------------|----------- Out of critical section | 5.7852s [1]: Tests performed on an i7-7700HQ with 16GB of RAM and SSD, running Manjaro Linux. [2]: chromium’s repo at commit 03ae96f (“Add filters testing at DSF=2”, 04-06-2019), after a 'git gc' execution. Signed-off-by: Matheus Tavares <matheus.bernardino@usp.br> Signed-off-by: Junio C Hamano <gitster@pobox.com>

Matheus Tavares committed Jan 15, 2020 at 23:39 UTC 70a9fef2402ff22db7691c6e3134943e1a4c1e1a
1 file changed +4 -4
builtin/grep.c
+4 -4
@@ -92,8 +92,11 @@ static pthread_cond_t cond_result;
92
93 static int skip_first_line;
94
95 -static void add_work(struct grep_opt *opt, const struct grep_source *gs)
95 +static void add_work(struct grep_opt *opt, struct grep_source *gs)
96 {
97 + if (opt->binary != GREP_BINARY_TEXT)
98 + grep_source_load_driver(gs, opt->repo->index);
99 +
100 grep_lock();
101
102 while ((todo_end+1) % ARRAY_SIZE(todo) == todo_done) {
@@ -101,9 +104,6 @@ static void add_work(struct grep_opt *opt, const struct grep_source *gs)
104 }
105
106 todo[todo_end].source = *gs;
104 - if (opt->binary != GREP_BINARY_TEXT)
105 - grep_source_load_driver(&todo[todo_end].source,
106 - opt->repo->index);
107 todo[todo_end].done = 0;
108 strbuf_reset(&todo[todo_end].out);
109 todo_end = (todo_end + 1) % ARRAY_SIZE(todo);